import java.util.Scanner;

public class Array {

	public static Scanner scnr = new Scanner(System.in);	

	
	
	private static int readInt() {
		int number = 0;
		
		number = scnr.nextInt();
		
		return number;
	}

	
	/**
	 * 
	 * This method displays the contents of the the array
	 * <b>arr</b>.
	 * 
	 * @param arr	The array to be filled with integers
	 * @param prompt The prompt to the user for each integer
	 */
	public static void showArray(int[] arr, int count) {
		System.out.println("Array has " + count + " elements");
		System.out.print("array = {");
		
		for ( int i = 0 ; i < count ; i++ ) {
			System.out.print(arr[i] + " ");
		}
		System.out.println("}");
	}

	
	/**
	 * 
	 * This method will read a list of intergers into the array 
	 * <code>arr</code> prompting the user for each integer with
	 *
	 * the <code>prompt</code> text. Input into the array is terminated
	 * when a <b>-1</b> is encountered.
	 * 
	 * @param arr	The array to be filled with integers
	 * @param prompt The prompt to the user for each integer
	 * 
	 * @return This method returns the count of integers read into the array.
	 * 
	 */
	public static int readArray(int[] arr, String prompt) {
		int count = 0;
		int index = 0;
		
		while (index < arr.length) {
			System.out.println(prompt);
			int input = readInt();
			
			if (input != -1) {
				arr[index++] = input;
				count++;
			} else {
				break;
			}
		}
		
		return count;
	}

	
	/**
	 * 
	 * This method searches for the maximum element in the array making sure
	 * to search only <b>count</b> elements in the array.
	 * 
	 * @param arr The array to be searched for maximum elements
	 * @param count The count of valid elements in the array
	 * 
	 * @return the maximum element in the array
	 * 
	 */
	public static int maxElement(int[] arr, int count) {
		return maxElement(arr, 0, count-1);
	}
	
	/**
	 * 
	 * This method returns the index of the first maximum element in
	 * the array <b>arr</b>
	 * 
	 * @param arr The array to be searched for maximum elements
	 * @param count The count of valid elements in the array
	 * 
	 * @return the index of the first maximum element in the array
	 */
	public static int indexOfMaxElement(int[] arr, int count) {
		return indexOfMaxElement(arr, 0, count-1);
	}
	

	/**
	 * 
	 * This method will search for and return the maximum element in the array
	 * <b>arr</b> starting at index <b>firstIndex</b> and ending at index
	 * <b>lastIndex</b>
	 * 
	 * @param arr 	The array that will be searched for the maximum element
	 * @param firstIndex	The index at which to start the search
	 * @param lastIndex		The index at which to end the search
	 * 
	 * @return The maximum element found in the index range
	 * 
	 */
	public static int maxElement(int[]arr, int firstIndex, int lastIndex) {
		int max = -1;
		
		for ( int i = firstIndex ; i <= lastIndex ; i++ ) {
			if (arr[i] > max) {
				max = arr[i];
			}
		}
		
		return max;
	}
	
	/**
	 * 
	 * This method will return the index of the first maximum element in the range
	 * <b>firstIndex</b> to <b>lastIndex</b>
	 * 
	 * @param arr 	The array that will be searched for the maximum element
	 * @param firstIndex	The index at which to start the search
	 * @param lastIndex		The index at which to end the search
	 * 
	 * @return the index of the first maximum element in the range <b>firstIndex</b> to
	 * <b>lastIndex</b>
	 * 
	 */
	public static int indexOfMaxElement(int[] arr, int firstIndex, int lastIndex) {
		int index = -1;
		
		int max = maxElement(arr, firstIndex, lastIndex);
		for ( int i = firstIndex ; i <= lastIndex ; i++ ) {
			if (arr[i] == max) {
				index = i;
				break;
			}
		}
		
		return index;
	}
	
	
	/*
	public static void reverseArray(int[] arr, int count) {
		int[] temp = new int[arr.length];
		
		int tempIndex = 0;
		
		for ( int i = count-1 ; i >= 0 ; i-- ) {
			temp[tempIndex++] = arr[i];
		}
		
		for ( int i = 0 ; i < count ; i++ ) {
			arr[i] = temp[i];
		}
		
	}
	*/
	
	/**
	 * 
	 * This method reverses the first <b>count</b> elements in 
	 * array <b>arr</b>.
	 * 
	 * @param arr The array to be reversed
	 * @param count The count of elements to be reversed.
	 * 
	 */
	public static void reverseArray(int[] arr, int count) {
		int i = 0;
		int j = count - 1;
		int temp;
		
		while (j > i) {
			temp = arr[i];
			arr[i] = arr[j];
			arr[j] = temp;
			i++;
			j--;
		}
	}

	/**
	 * 
	 * This method is used to rotate the array clockwise one position.
	 * In other words, all elements over one space to the right and
	 * then move the last element to the first position.
	 * 
	 * @param arr The array that will be rotated once
	 * @param count The count of elements in the array
	 * 
	 */
	public static void rotateOnce(int[] arr, int count) {
		int temp = arr[count-1];
		
		for ( int i = (count - 1) ; i > 0 ; i-- ) {
			arr[i] = arr[i-1];
		}
		arr[0] = temp;
	}
	
	
	/**
	 * 
	 * This method is used to rotate the specified array <b>rotations</b>
	 * times. This method will call <b>rotateOnce()</b> multiple times. 
	 * 
	 * @param arr The array that will be rotated
	 * @param count The count of elements in the array
	 * @param rotations The number of times to rotate the array
	 * 
	 * 
	 */
	public static void rotateMultiple(int[] arr, int count, int rotations) {
		for (int i = 0 ; i < rotations ; i++ ) {
			rotateOnce(arr, count);
		}
	}
	
	/**
	 * 
	 * This method is used to insert the specified element into the array at the
	 * specified location.
	 * 
	 * Note: If the array is full, this method will print an error message and return
	 * without performing the insertion.
	 * 
	 * @param arr The array that will be used to perform the insertion
	 * @param count The count of elements in the array
	 * @param element The element to be inserted
	 * @param location The location where the element is inserted
	 * 
	 */
	public static void insertElementAt(int[] arr, int count, int element, int location) {
		
		if (arr.length == count) {
			System.out.println("Array is full, I cannot insert into this array");
			return;
		}
		
		for ( int i = count ; i > location ; i-- ) {
			arr[i] = arr[i-1];
		}
		arr[location] = element;
	}
}
